function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const {
  exec
} = require('child_process');

module.exports = class Git {
  constructor(debug = false) {
    _defineProperty(this, "debug", void 0);

    this.debug = debug;
  }
  /**
   * Log commands
   *
   * @param   {Array} params console.log parameters
   *
   * @returns {void}
   */


  log(...params) {
    if (this.debug) {
      console.log(...params);
    }
  }
  /**
   * Get current branch
   *
   * @returns {Promise} Promise that resolves to branch name
   */


  getCurrentBranch() {
    const cmd = 'git rev-parse --abbrev-ref HEAD';
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout.trim());
        } else {
          reject(stderr);
        }
      });
    });
  }
  /**
   * Check is there are staged files
   *
   * @returns {Promise} Promise that resolves to boolean true or false
   */


  hasStagedFiles() {
    const cmd = 'git diff --cached --name-only';
    this.log('CMD:', cmd);
    return new Promise(resolve => {
      exec(cmd, (error, stdout) => {
        if (!error && stdout.trim() !== '') {
          resolve(true);
        } else {
          resolve(false);
        }
      });
    });
  }
  /**
   * Get status
   *
   * @returns {Promise} Promise that resolves to git status
   */


  getStatus() {
    const cmd = 'git status';
    this.log('CMD:', cmd);
    return new Promise(resolve => {
      exec(cmd, (error, stdout) => {
        if (!error) {
          resolve(stdout);
        } else {
          resolve('Unable to get git status!');
        }
      });
    });
  }
  /**
   * Fetch origin
   *
   * @returns {Promise} Promise
   */


  fetchOrigin() {
    const cmd = 'git fetch origin';
    this.log('CMD:', cmd);
    return new Promise(resolve => {
      exec(cmd, error => {
        if (!error) {
          resolve('Origin fetched');
        } else {
          resolve('Unable to fetch origin!');
        }
      });
    });
  }
  /**
   * Check remote branch
   *
   * @param   {string}  branchName Name of the branch
   *
   * @returns {Promise}            Promise that resolves to true/false
   */


  hasRemoteBranch(branchName) {
    const cmd = `git ls-remote --heads origin ${branchName}`;
    this.log('CMD:', cmd);
    return new Promise(resolve => {
      exec(cmd, (error, stdout) => {
        if (!error) {
          resolve(stdout.includes(`refs/heads/${branchName.trim()}`));
        } else {
          resolve('Unable to fetch origin!');
        }
      });
    });
  }
  /**
   * Check if local is in sync with remote
   *
   * @param   {string}  branchName Name of the branch
   *
   * @returns {Promise}            Promise that resolves to true/false
   */


  isInSync(branchName) {
    const cmd = 'git branch -r --contains HEAD';
    this.log('CMD:', cmd);
    return new Promise(resolve => {
      exec(cmd, (error, stdout) => {
        if (!error) {
          resolve(stdout.includes(branchName.trim()));
        } else {
          resolve('Unable to check remote branch!');
        }
      });
    });
  }
  /**
   * Stage files
   *
   * @param   {string}  arg git add argument
   *
   * @returns {Promise}     Promise that resolves to stage result
   */


  stage(arg) {
    const cmd = `git add ${arg}`;
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          reject(stderr);
        }
      });
    });
  }

  stageIfExists(arg) {
    const cmd = `ls ${arg}`;
    return new Promise(resolve => {
      exec(cmd, (error, stdout) => {
        if (!error) {
          resolve(stdout);
          this.stage(arg);
        } else {
          resolve();
        }
      });
    });
  }
  /**
   * Commit changes from staging
   *
   * @param   {string}  msg Commit message
   *
   * @returns {Promise}     Promise that resolves to commit result
   */


  commit(msg) {
    /* eslint-disable no-useless-escape */
    const cmd = `git commit -m '${msg.replace(new RegExp('\'', 'g'), '\'\\\'\'').replace(new RegExp('\`', 'g'), '\'\\\`\'')}'`;
    /* eslint-enable no-useless-escape */

    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          reject(stderr);
        }
      });
    });
  }
  /**
   * Push branch to remote repo (origin)
   *
   * @param   {string}  branch       Branch name
   * @param   {string}  remoteBranch Branch name
   *
   * @returns {Promise}              Promise that resolves to push result
   */


  push(branch, remoteBranch = null) {
    const cmd = `git push origin ${branch}:${remoteBranch || branch}`;
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          this.log(stderr);
          reject(stderr);
        }
      });
    });
  }
  /**
   * Get tags
   *
   * @returns {Promise} Promise that resolves to tags list sorted by date
   */


  getTags() {
    const cmd = 'git for-each-ref refs/tags --sort=-taggerdate --format="%(refname)"';
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          const refTags = stdout.split('\n');
          refTags.pop();
          resolve(refTags.map(tag => tag.split('/')[2].trim()));
        } else {
          reject(stderr);
        }
      });
    });
  }
  /**
   * Get git log
   *
   * @param   {string} from    Starting ref
   * @param   {string} to      Ending ref
   * @param   {srting} options Additional git parameters
   *
   * @returns {Promise}        Promise that resolves to git log
   */


  getLog(from, to, options) {
    let cmd = 'git log';

    if (from && to) {
      cmd += ` ${from}..${to}`;
    } else if (!from && to) {
      cmd += ` ${to}`;
    }

    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      cmd += options ? ` ${options}` : '';
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          reject(stderr);
        }
      });
    });
  }
  /**
   * Add tag
   *
   * @param   {string}  tagname Name of the tag
   *
   * @returns {Promise}         Promise that resolves to tag add result
   */


  addTag(tagname) {
    const cmd = `git tag -a '${tagname}' -m '${tagname}'`;
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          reject(stderr);
        }
      });
    });
  }
  /**
   * Creates a branch for the provided branch name
   *
   * @param   {string}  branchName the name of the branch being created
   *
   * @returns {Promise} promise which gets resolved once the branch is created or rejected if an error occured
   */


  createBranch(branchName) {
    const cmd = `git checkout -b '${branchName}'`;
    this.log('CMD:', cmd);
    return new Promise((resolve, reject) => {
      exec(cmd, (error, stdout, stderr) => {
        if (!error) {
          resolve(stdout);
        } else {
          reject(stderr);
        }
      });
    });
  }

};