function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const gitLabClient = require('gitlab');
/**
 * GitLab wrapper
 */


module.exports = class GitLab {
  /**
   * Constructor
   *
   * @param {Object} config GitLab workflow config
   */
  constructor(config) {
    _defineProperty(this, "gitClient", void 0);

    _defineProperty(this, "mergeRequestsProperty", void 0);

    _defineProperty(this, "project", void 0);

    _defineProperty(this, "config", void 0);

    _defineProperty(this, "gitWebUrl", void 0);

    this.gitWebUrl = 'web_url';
    this.mergeRequestsProperty = 'merge_requests';
    this.gitClient = gitLabClient(config);
    this.config = config;
  }
  /**
   * Search for project by name
   *
   * @param   {string}  projectName      Project name
   * @param   {number}  page             Page number
   *
   * @returns {Promise}                  Promise that resolves to project object
   */


  localSearchForProjectPaged(projectName, page) {
    return new Promise((resolve, reject) => {
      this.gitClient.projects.search(projectName, {
        simple: true,
        per_page: 50,
        page
      }, (projects, err) => {
        if (!err) {
          resolve(projects);
        } else {
          reject(err);
        }
      });
    });
  }
  /**
   * Search for project by name
   *
   * @param   {string}  projectName      Project name
   * @param   {string}  projectNamespace Project namespace
   *
   * @returns {Promise}                  Promise that resolves to project object
   */


  async localSearchForProject(projectName, projectNamespace) {
    let page = 1;
    let project = null;

    do {
      const data = await this.localSearchForProjectPaged(projectName, page);

      if (data.length === 0) {
        throw new Error('Project not found!');
      } else {
        project = data.find(proj => proj.name === projectName && (projectNamespace === '' || proj.namespace.path === projectNamespace));
      }

      page++;
    } while (!project);

    return project;
  }
  /**
   * Get WebUrl path to current project
   *
   * @returns {string} web url path to current project
   */


  getWebUrl() {
    return this.project ? this.project[this.gitWebUrl] : '';
  }
  /**
   * Initialize current GitLab object
   *
   * @returns {mixed} error message or true
   */


  async init() {
    try {
      this.project = await this.localSearchForProject(this.config.projectName, this.config.projectNamespace);
    } catch (err) {
      return err;
    }

    return true;
  }
  /**
   * Get userId based on user name
   *
   * @param   {string}   name Username
   *
   * @returns {Promise}      Promise that resolves to user id
   */


  getUserId(name) {
    return new Promise((resolve, reject) => {
      this.gitClient.users.search(name, (data, err) => {
        if (!err) {
          resolve(data[0].id);
        } else {
          reject(err);
        }
      });
    });
  }
  /**
   * Create merge request in GitLab
   *
   * @param   {string}  sourceBranch Source branch name
   * @param   {string}  targetBranch Target branch name
   * @param   {string}  title        MR title
   *
   * @returns {Promise}              Promise that resolves to status data
   */


  async createMergeRequest(sourceBranch, targetBranch, title) {
    return new Promise((resolve, reject) => {
      this.gitClient.projects[this.mergeRequestsProperty].add(this.project.id, sourceBranch, targetBranch, undefined, title, (data, err) => {
        if (!err) {
          resolve(data);
        } else {
          reject(err);
        }
      });
    });
  }

};