function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const JiraClient = require('jira-connector');

const axios = require('axios');
/**
 * JIRA connection wrapper
 */


module.exports = class Jira {
  /**
   * Constructor
   *
   * @param {Object} config JIRA workflow config
   */
  constructor(config) {
    _defineProperty(this, "jiraAuthTypePath", void 0);

    _defineProperty(this, "config", void 0);

    _defineProperty(this, "username", void 0);

    _defineProperty(this, "client", void 0);

    _defineProperty(this, "transitions", void 0);

    this.jiraAuthTypePath = 'basic_auth';
    this.config = config;
    this.username = this.config[this.jiraAuthTypePath].username;
    this.client = new JiraClient(this.config);
  }
  /**
   * Local get JIRA issue function
   *
   * @param   {string}  issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {Promise}          Promise that resolves to JIRA issue object
   */


  localGetIssue(issueKey) {
    return new Promise((resolve, reject) => {
      this.client.issue.getIssue({
        issueKey
      }, (e, data) => {
        if (!e) {
          resolve(data);
        } else {
          reject(e);
        }
      });
    });
  }
  /**
   * Get issue transitions
   *
   * @param   {string}  issueKey Issue key
   *
   * @returns {Promise}          Promise that resolves to transitions json
   */


  getTransitions(issueKey) {
    return new Promise((resolve, reject) => {
      if (this.transitions) {
        resolve(this.transitions);
      } else {
        this.client.issue.getTransitions({
          issueKey
        }, (e, data) => {
          if (!e && data.transitions) {
            this.transitions = data.transitions.map(transition => ({
              value: transition.id,
              name: transition.name
            }));
            this.transitions.unshift({
              value: 'N/A',
              name: 'Do not update Jira!'
            });
            resolve(this.transitions);
          } else {
            reject(e);
          }
        });
      }
    });
  }
  /**
   * Get JIRA issue
   *
   * @param   {string} issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {Object}          JIRA issue object
   */


  async getIssue(issueKey) {
    try {
      return await this.localGetIssue(issueKey);
    } catch (err) {
      return {}.hasOwnProperty.call(err, 'errorMessages') ? err.errorMessages[0] : 'Error';
    }
  }
  /**
   * Check if issue exists in JIRA
   *
   * @param   {string}  issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {boolean}          true if issue exists in JIRA
   */


  async hasIssue(issueKey) {
    try {
      return (await this.localGetIssue(issueKey)) !== null;
    } catch (err) {
      return {}.hasOwnProperty.call(err, 'errorMessages') ? err.errorMessages[0] : 'Error';
    }
  }
  /**
   * Assign JIRA to current user
   *
   * @param   {string}  issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {Promise}          Promise that resolves to issue key
   */


  assignIssueToMe(issueKey) {
    return new Promise((resolve, reject) => {
      this.client.issue.assignIssue({
        issueKey,
        assignee: this.username
      }, e => {
        if (!e) {
          resolve(issueKey);
        } else {
          reject(e);
        }
      });
    });
  }
  /**
   * Set issue status
   *
   * @param   {string}  status   New JIRA status
   * @param   {string}  issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {Promise}          Promise that resolves to issue key
   */


  setIssueStatusTo(status, issueKey) {
    return new Promise((resolve, reject) => {
      if (status !== '0') {
        this.client.issue.transitionIssue({
          issueKey,
          transition: status
        }, e => {
          if (!e) {
            resolve(issueKey);
          } else {
            console.log('rejected: setIssueStatusTo', e);
            reject(e);
          }
        });
      } else {
        resolve(issueKey);
      }
    });
  }
  /**
   * Update Issue version
   *
   * @param   {Object} version Version to add
   * @param   {Object} issue   Issue object
   *
   * @returns {Promise}        Promise that resolves to issue object
   */


  updateIssueVersion(version, issue) {
    const newIssue = {
      fields: {
        fixVersions: [version]
      }
    };
    return new Promise((resolve, reject) => {
      this.client.issue.editIssue({
        issueKey: issue.key,
        issue: newIssue
      }, (e, data) => {
        if (!e) {
          resolve(data);
        } else {
          reject(e);
        }
      });
    });
  }
  /**
   * Local function to add comment
   *
   * @param   {string} comment  Comment string
   * @param   {string} issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {Promise}         Promise that resoles to true
   */


  localAddComment(comment, issueKey) {
    return new Promise((resolve, reject) => {
      this.client.issue.addComment({
        issueKey,
        comment: {
          body: comment
        }
      }, e => {
        if (!e) {
          resolve(true);
        } else {
          reject(e);
        }
      });
    });
  }
  /**
   * Add comment to JIRA issue
   *
   * @param   {string}  comment  Comment string
   * @param   {string}  issueKey JIRA issue key (eg. SBKS-84)
   *
   * @returns {mixed}            true on success
   */


  async addComment(comment, issueKey) {
    if (comment !== '') {
      try {
        return await this.localAddComment(comment, issueKey);
      } catch (err) {
        return err;
      }
    }

    return false;
  }
  /**
   * Get gurrent username
   * @returns {string} current username
   */


  getUsername() {
    return this.username;
  }
  /**
   * Assign issue to current user if not assigned
   * @param   {Object} issue    JIRA issue object
   * @param   {string} issueKey JIRA issue key (eg. SBKS-84)
   * @param   {string} me       User to assign issue to
   * @returns {void}
   */


  async assignIssueToMeIfNotAssigned(issue, issueKey, me) {
    if (issue.fields) {
      const assignee = issue.fields.assignee || {};

      if (assignee.key !== me) {
        await this.assignIssueToMe(issueKey);
      }
    } else {
      await this.assignIssueToMe(issueKey);
    }
  }
  /**
   * Check Jira connection
   *
   * @returns {void}
   */


  async checkConnection() {
    return new Promise((resolve, reject) => {
      axios.get(`https://${this.config.host}/rest/api/2/mypermissions`, {
        headers: {
          Authorization: `Basic ${this.config[this.jiraAuthTypePath].base64}`
        }
      }).then(() => resolve()).catch(error => {
        /*eslint quote-props: ["error", "consistent"]*/
        const {
          status
        } = error.response || {
          status: 404
        };
        const errorMessages = {
          '400': 'Cannot connect to Jira. Check your host in .workflowrc!',
          '401': 'Cannot authenticate on Jira. Check your username and password in ~/.workflowrc!',
          '403': 'Cannot authenticate on Jira. A CAPTCHA is being requested. ' + 'Please try to login on your browser.',
          '404': 'Cannot connect to Jira. Check your host in .workflowrc!',
          default: 'Cannot authenticate on Jira. Check your configuration in .workflowrc and ~/.workflowrc'
        };

        if (Object.prototype.hasOwnProperty.call(errorMessages, status)) {
          reject(new Error(errorMessages[status]));
        } else {
          reject(new Error(errorMessages.default));
        }
      });
    });
  }

};