const fs = require('fs');

const path = require('path');

const wrap = require('word-wrap');

const gitRev = require('git-rev');

const {
  spawn
} = require('child_process');

const merge = require('lodash/merge');

const pkg = require('../../package.json');

const maxLineWidth = 300;
const wrapOptions = {
  trim: true,
  newline: '\n',
  indent: '',
  width: maxLineWidth
};
const utf8Support = process.env.LANG ? process.env.LANG.includes('UTF-8') : false;
const charMap = {
  sparkles: '\u2728 ',
  ok: '\u2713',
  err: '\u2717'
  /**
   * Generate UTF-8 sign of fallback to ASCII
   *
   * @param   {string} chr Name of utf8
   * @param   {string} def Default value
   *
   * @returns {string}     utf8 string or ascii default
   *
   * @constructor
   */

};

module.exports.u = function u(chr, def = '') {
  if (utf8Support && charMap.hasOwnProperty(chr)) {
    // eslint-disable-line no-prototype-builtins
    return charMap[chr];
  }

  return def;
};
/**
 * Create git commit message from answers
 *
 * @param   {Object} clients       Clients
 * @param   {Object} answers       Object with answers
 * @param   {Object} commitAnswers Object with answers for commit
 *
 * @returns {string}               Formatted commit message
 */


module.exports.createGitCommitMessageFromAnswers = async function createGitCommitMessageFromAnswers(clients, answers, commitAnswers) {
  const {
    jira
  } = clients;
  const {
    jiraStatus,
    issueKey
  } = answers;
  const {
    typeOfWork,
    subject,
    body
  } = commitAnswers;
  const transitions = await jira.getTransitions(issueKey);
  const objJiraStatus = transitions.find(status => status.value === jiraStatus);
  const textHead = `${typeOfWork}(${issueKey}): ${subject}`.slice(0, maxLineWidth);
  const textBody = wrap(body, wrapOptions);
  const textJira = `Jira status: ${objJiraStatus.name}`;
  return `${textHead}` + `${textBody ? `\n\n${textBody}` : ''}` + `\n\n${textJira}`;
};
/**
 * Create merge title
 *
 * @param   {Object}  issue       JIRA issue
 * @param   {boolean} doTag       If MR should contain special markup to make a tag
 * @param   {string}  versionType Version type change (patch, minor, major)
 *
 * @returns {string}              Merge request title
 */


module.exports.createMergeTitle = function createMergeTitle(issue, doTag, versionType) {
  return `${issue.key}: ${issue.fields.summary} (${issue.fields.issuetype.name})` + `${doTag ? ` versionType:${versionType}` : ''}`;
};
/**
 * Generate Jira status comment
 *
 * @param   {string} issueKey     JIRA key (eg. SBSK-84)
 * @param   {string} status       Item status
 * @param   {string} user         Item username
 * @param   {Array}  statusesList List of Jira statuses
 *
 * @returns {string}              Formatted JIRA comment
 */


module.exports.generateJiraStatusComment = function generateJiraStatusComment(issueKey, status, user, statusesList) {
  let comment = '';

  if (status !== '0') {
    comment = `The issue(${issueKey}) has been updated with status: ${statusesList.filter(item => item.value === status)[0].name} by [~${user}] via CLI`;
    console.log(comment);
  }

  return comment;
};
/**
 * Get current git branch name
 *
 * @returns {Promise} Promise that resolves to branch name or empty
 */


module.exports.getIssueKeyFromBranch = function getIssueKeyFromBranch() {
  return new Promise(resolve => {
    gitRev.branch(branch => {
      const spl = branch.split('/');

      if (spl.length === 2) {
        resolve(spl[1]);
      }

      resolve('');
    });
  });
};
/**
 * Local read .workflowrc function
 *
 * @param   {string} workflowrcPath Path to workflowrc file
 *
 * @returns {Object}                Workflowrc file in JSON object
 */


function readWorkflow(workflowrcPath) {
  let workflowrc;

  try {
    workflowrc = JSON.parse(fs.readFileSync(workflowrcPath, 'utf8'));
  } catch (e) {
    workflowrc = undefined;
  }

  return workflowrc;
}
/**
 * Get empty string if input not defined
 *
 * @param   {strnig} input Input string
 *
 * @returns {string}       input or empty
 */


const defaultToEmptyString = input => input || '';
/**
 * Read configuration in workflowrc
 * @returns {Obejct} Configuration
 */


module.exports.readWorkflowConfig = function readWorkflowConfig() {
  const workflowrcGlobal = readWorkflow(path.join(defaultToEmptyString(process.env.HOME), '.workflowrc'));
  const workflowrcLocal = readWorkflow(path.join(defaultToEmptyString(process.env.PWD), '/.workflowrc'));
  const workflowrcLocalMono = readWorkflow(path.join('../../../', '.workflowrc')) || {};

  if (!workflowrcLocal && !workflowrcLocalMono) {
    throw new Error('error reading ./workflowrc file from ./ or ../../../');
  }

  if (!workflowrcGlobal && !workflowrcLocal) {
    throw new Error('Please configure a local and global .workflowrc file');
  }

  const workflowrc = merge({}, workflowrcGlobal, workflowrcLocal, workflowrcLocalMono);
  return workflowrc;
};
/**
 * Execute command
 *
 * @param   {string}  cmd    Command to be executed
 * @param   {string}  params Parameters
 *
 * @returns {Promise}        Promise that resolves to exit code
 */


module.exports.execLive = function execLive(cmd, ...params) {
  return new Promise((resolve, reject) => {
    const ecmd = spawn(cmd, params);
    ecmd.stdout.on('data', data => {
      console.log(data.toString());
    });
    ecmd.stderr.on('data', data => {
      console.log(data.toString());
    });
    ecmd.on('exit', code => {
      console.log(`Command '${cmd}${params.reduce((a, b) => `${a} ${b}`, '')}'` + ` exited with code ${code}`);

      if (code) {
        reject(code);
      } else {
        resolve(code);
      }
    });
  });
};
/**
 * Generate greeting
 *
 * @returns {void}
 */


module.exports.greeitng = function greeitng() {
  console.log(`
 ▄  █ ▄███▄   █    █    ████▄   ${u('sparkles')}
█   █ █▀   ▀  █    █    █   █
██▀▀█ ██▄▄    █    █    █   █
█   █ █▄   ▄▀ ███▄ ███▄ ▀████
   █  ▀███▀       ▀    ▀   
  ▀           dev-workflow version ${pkg.version}
`);
};