const JiraApi = require('./common/jira');

const GitLab = require('./common/gitlab');

const Git = require('./common/git');

const {
  readWorkflowConfig
} = require('./common/utils');

const fs = require('fs');

const readPkg = require('read-pkg-up');

const truncate = require('cli-truncate');

const wrap = require('wrap-ansi');

const pad = require('pad');

const fuse = require('fuse.js');

const homeDir = require('home-dir');

const branch = require('git-branch');

const types = require('../lib/types');

const defaultConfig = {
  types,
  symbol: false
};
let jiraName = '';
let commitTitle = '';
let commitBody = '';
let commitFooter = '';
const workflowrc = readWorkflowConfig(true);
const clients = {
  jira: new JiraApi(workflowrc.jira),
  git: new Git(),
  gitlab: new GitLab(workflowrc.gitlab)
};

async function updateJira(issueKey) {
  const {
    jira
  } = clients;
  const jiraStatus = workflowrc.reviewTransitionValue;

  if (jiraStatus !== 'N/A') {
    const issue = await jira.getIssue(issueKey);
    await jira.assignIssueToMeIfNotAssigned(issue, issueKey, jira.getUsername());
    await jira.setIssueStatusTo(jiraStatus, issueKey);
    const {
      url,
      projectNamespace,
      projectName
    } = workflowrc.gitlab;
    const GITLAB_MR_URL = `${url}/${projectNamespace}/${projectName}/merge_requests`;
    await jira.addComment(`New commit: ${GITLAB_MR_URL}`, issueKey);
  }
}

async function submitMergeRequest() {
  const sourceBranch = branch.sync();

  if (sourceBranch === 'master') {
    console.log(`ERROR - don't push directly to master. Please create a feature branch`);
    return;
  }

  await clients.git.push(sourceBranch);
  console.log(`${sourceBranch} pushed`);
  await clients.gitlab.init();
  clients.gitlab.createMergeRequest(sourceBranch, 'master', commitTitle);
  console.log('merge request created');
}

function getEmojiChoices({
  types,
  symbol
}) {
  const maxNameLength = types.reduce((maxLength, type) => type.name.length > maxLength ? type.name.length : maxLength, 0);
  return types.map(choice => ({
    name: `${pad(choice.name, maxNameLength)}  ${choice.emoji}  ${choice.description}`,
    value: symbol ? choice.emoji : choice.code,
    code: choice.code
  }));
}

function loadConfig() {
  const getConfig = obj => obj && obj.config && obj.config['cz-jira-emoji'];

  return readPkg().then(({
    pkg
  }) => {
    const config = getConfig(pkg);
    if (config) return config;
    return new Promise((resolve, reject) => {
      fs.readFile(homeDir('.czrc'), 'utf8', (err, content) => {
        if (err) reject(err);
        const czrc = content && JSON.parse(content) || null;
        resolve(getConfig(czrc));
      });
    });
  }).then(config => Object.assign({}, defaultConfig, config)).catch(() => defaultConfig);
}
/**
 * Create inquier.js questions object trying to read `types` and `scopes` from the current project
 * `package.json` falling back to nice default :)
 *
 * @param {Object} config Result of the `loadConfig` returned promise
 * @return {Array} Return an array of `inquier.js` questions
 * @private
 */


function createQuestions(config) {
  jiraName = workflowrc['default-jira-name'] || 'OREO';
  const typeChoices = getEmojiChoices(config);
  const scopeChoices = workflowrc.scopes.map(scope => ({
    name: scope
  }));
  const fuzzy = new fuse(typeChoices, {
    shouldSort: true,
    threshold: 0.4,
    location: 0,
    distance: 100,
    maxPatternLength: 32,
    minMatchCharLength: 1,
    keys: ['name', 'code']
  });
  const scopeFuzzy = new fuse(scopeChoices, {
    shouldSort: true,
    threshold: 0.4,
    location: 0,
    distance: 100,
    maxPatternLength: 32,
    minMatchCharLength: 1,
    keys: ['name']
  });
  const branchTicketMatches = branch.sync().match(/\w+-\d+/g);
  const branchTicket = branchTicketMatches ? branchTicketMatches.join('') : '';
  const questions = [{
    type: 'autocomplete',
    name: 'type',
    message: "Select the type of change you're committing:",
    source: (answersSoFar, query) => {
      return Promise.resolve(query ? fuzzy.search(query) : typeChoices);
    }
  }, {
    type: workflowrc.scopes ? 'autocomplete' : 'input',
    name: 'scope',
    message: 'Specify work scope:',
    source: (answersSoFar, query) => {
      return Promise.resolve(query ? scopeFuzzy.search(query) : scopeChoices);
    }
  }, {
    type: 'input',
    name: 'subject',
    message: 'Write a short description:'
  }, {
    type: 'input',
    name: 'tickets',
    default: branchTicket,
    message: jiraName ? 'List related JIRA tickets (12, ' + jiraName + '-318):' : 'List related tickets (OREO-12, SDP-28):'
  }, {
    type: 'input',
    name: 'body',
    message: '(Optional) Provide a longer description:'
  }];

  if (workflowrc.versioning) {
    questions.push({
      type: 'list',
      name: 'versioning',
      message: 'Want to deploy?',
      choices: ['no'].concat(workflowrc.versioning)
    });
  }

  return questions;
}
/**
 * Format the git commit message from given answers.
 *
 * @param {Object} answers Answers provide by `inquier.js`
 * @return {String} Formated git commit message
 */


function format(answers) {
  // parentheses are only needed when a scope is present
  const scope = answers.scope ? '(' + answers.scope.trim() + ') ' : '';
  const ticketNumbers = answers.tickets.match(/(^|[ ])\d+/g) || [];
  const ticketsNumbersWithDefaultName = ticketNumbers.map(tickerNumber => jiraName + '-' + tickerNumber.trim());
  const fullNameTickets = answers.tickets.match(/\w+-\d+/g) || [];
  const ticketsArr = ticketsNumbersWithDefaultName.concat(fullNameTickets); // update Jira status

  ticketsArr.forEach(ticketNumber => updateJira(ticketNumber)); // join to

  const tickets = ticketsArr.join(' ');
  const versioning = answers.versioning === 'no' ? '' : ' versionType:' + answers.versioning; // build head line and limit it to 100

  const head = truncate(answers.type + ' ' + scope + tickets + ' ' + answers.subject.trim() + versioning, 100); // wrap body at 100

  const body = wrap(answers.body, 100);
  const footer = (answers.tickets.match(/\w+-\d+/g) || []).map(tickets => `${tickets}`).join('\n');
  commitTitle = head;
  commitBody = body;
  commitFooter = footer;
  return [head, body, footer].join('\n\n').trim();
}
/**
 * Export an object containing a `prompter` method. This object is used by `commitizen`.
 *
 * @type {Object}
 */


module.exports = {
  prompter: function (cz, commit) {
    cz.prompt.registerPrompt('autocomplete', require('inquirer-autocomplete-prompt'));
    loadConfig().then(createQuestions).then(cz.prompt).then(format).then(commit).then(submitMergeRequest);
  }
};